use std::sync::Arc;

use axum::extract::State;
use axum::http::StatusCode;
use axum::{Json, Router};
use axum::routing::get;
use crate::base::controller::BaseController;

use super::model::User;
use super::service::UserService;

pub struct UserController {
    base_controller: BaseController<User, UserService>,
}

impl UserController {
    pub fn new(service: UserService) -> Self {
        Self {
            base_controller: BaseController::new(service),
        }
    }

    pub fn routes(state: Arc<UserController>) -> Router {
        Router::new()
            .route("/users", get(Self::get_users))
            .with_state(state)
    }

    async fn get_users(
        State(controller): State<Arc<UserController>>,
    ) -> Result<Json<Vec<User>>, (StatusCode, Json<String>)> {
        match controller.base_controller.get_all().await {
            Ok(users) => {
                // Wrap the users in Json, which implements IntoResponse
                Ok(Json(users))
            }
            Err(err) => {
                // In case of error, return StatusCode with Json error message
                Err((StatusCode::INTERNAL_SERVER_ERROR, Json(err.to_string())))
            }
        }
    }
}
